﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.UI;
using System.Reflection;
using System.Web;

[assembly: WebResource("AspNetAjaxExtensions.UpdatePanelFileUpload.js", "application/x-javascript")]

namespace AspNetAjaxExtensions
{
    [PersistChildren(false)]
    [ParseChildren(true)]
    [NonVisualControl]
    public class UpdatePanelFileUpload : Control
    {
        // ScriptManager members;
        private readonly static FieldInfo s_isInAsyncPostBackFieldInfo;
        private readonly static PropertyInfo s_pageRequestManagerPropertyInfo;

        // PageRequestManager members;
        private readonly static MethodInfo s_onPageErrorMethodInfo;

        static UpdatePanelFileUpload()
        {
            Type scriptManagerType = typeof(ScriptManager);
            s_isInAsyncPostBackFieldInfo = scriptManagerType.GetField(
                "_isInAsyncPostBack",
                BindingFlags.Instance | BindingFlags.NonPublic);
            s_pageRequestManagerPropertyInfo = scriptManagerType.GetProperty(
                "PageRequestManager",
                BindingFlags.Instance | BindingFlags.NonPublic);

            Assembly assembly = scriptManagerType.Assembly;
            Type pageRequestManagerType = assembly.GetType("System.Web.UI.PageRequestManager");
            s_onPageErrorMethodInfo = pageRequestManagerType.GetMethod(
                "OnPageError", BindingFlags.Instance | BindingFlags.NonPublic);
        }

        public static UpdatePanelFileUpload GetCurrent(Page page)
        {
            return page.Items[typeof(UpdatePanelFileUpload)] as UpdatePanelFileUpload;
        }

        private bool m_pageInitialized = false;

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (this.Page.Items.Contains(typeof(UpdatePanelFileUpload)))
            {
                throw new InvalidOperationException("One UpdatePanelFileUpload control per page.");
            }

            this.Page.Items[typeof(UpdatePanelFileUpload)] = this;

            this.Page.InitComplete += (sender, ea) =>
            {
                this.m_pageInitialized = true;
            };

            this.IsInUploadAsyncPostBack = FileUploadUtility.IsInUploadAsyncPostBack(this.Context);
            if (this.IsInUploadAsyncPostBack)
            {
                s_isInAsyncPostBackFieldInfo.SetValue(ScriptManager.GetCurrent(this.Page), true);
                this.Page.Error += (sender, ea) =>
                {
                    s_onPageErrorMethodInfo.Invoke(this.PageRequestManager, new object[] { sender, ea }); 
                };
            }
        }

        public bool IsInUploadAsyncPostBack { get; private set; }

        private bool m_enabled = true;

        public bool Enabled
        {
            get
            {
                return this.m_enabled;
            }
            set
            {
                if (this.m_pageInitialized)
                {
                    throw new InvalidOperationException("Cannot set the Enable property after page initialized.");
                }

                this.m_enabled = value;
            }
        }


        private string m_executeMethod;

        public string ExecuteMethod
        {
            get
            {
                return this.m_executeMethod;
            }
            set
            {
                if (this.m_pageInitialized)
                {
                    throw new InvalidOperationException("Cannot set the ExecuteMethod property after page initialized.");
                }

                this.m_executeMethod = value;
            }
        }

        private object m_pageRequestManager;

        private object PageRequestManager
        {
            get
            {
                if (this.m_pageRequestManager == null)
                {
                    this.m_pageRequestManager = s_pageRequestManagerPropertyInfo.GetValue(
                        ScriptManager.GetCurrent(this.Page), null);
                }

                return this.m_pageRequestManager;
            }
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            ScriptManager sm = ScriptManager.GetCurrent(this.Page);
            if (sm.IsInAsyncPostBack || !sm.EnablePartialRendering ||
                this.IsInUploadAsyncPostBack || !this.Enabled)
            {
                return;
            }

            if (String.IsNullOrEmpty(this.ExecuteMethod))
            {
                throw new ArgumentException("Please provide the ExecuteMethod.");
            }

            ScriptReference script = new ScriptReference(
                "AspNetAjaxExtensions.UpdatePanelFileUpload.js",
                this.GetType().Assembly.FullName);
            ScriptManager.GetCurrent(this.Page).Scripts.Add(script);

            if (!String.IsNullOrEmpty(this.ExecuteMethod))
            {
                this.Page.ClientScript.RegisterStartupScript(
                    this.GetType(),
                    "ExecuteMethod",
                    "AspNetAjaxExtensions.UpdatePanelIFrameExecutor._executeForm = " + this.ExecuteMethod + ";",
                    true);
            }
        }


    }
}
